@@ -216,7 +216,7 @@ TrackList* TrackListLoader::parse(char *data, size_t len)
   return tracks;
 }
 
-void TrackListLoader::check_album_license(guint32 album_id, sigc::slot<void, guint32, Glib::ustring> success_slot, sigc::slot<void,guint32> failure_slot)
+void TrackListLoader::check_album_license(guint32 album_id, sigc::slot<void, guint32, Glib::ustring, Glib::RefPtr<Gdk::Pixbuf>, double> success_slot, sigc::slot<void,guint32> failure_slot)
 {
   Glib::ustring url = String::ucompose("http://www.jamendo.com/en/album/%1",
                                        album_id);
@@ -230,7 +230,7 @@ void TrackListLoader::check_album_license(guint32 album_id, sigc::slot<void, gui
 
 void TrackListLoader::on_album_info_downloaded(Glib::RefPtr < Soup::Message > &msg,
                                                guint32 album_id,
-                                               sigc::slot<void,guint32, Glib::ustring> success, 
+                                               sigc::slot<void,guint32, Glib::ustring, Glib::RefPtr<Gdk::Pixbuf>, double> success, 
                                                sigc::slot<void,guint32> failure)
 {
   if (msg->property_status_code () == 200)
@@ -241,8 +241,24 @@ void TrackListLoader::on_album_info_downloaded(Glib::RefPtr < Soup::Message > &m
       const char *data = buffer.gobj ()->data;
       guint len = buffer.gobj ()->length;
       Glib::ustring license;
-      if (get_license(data, len, license))
-        success(album_id, license);
+      double rating = 0.0;
+      if (get_license_and_rating(data, len, license, rating))
+        {
+          std::map<Glib::ustring, Glib::RefPtr<Gdk::Pixbuf> >::iterator i;
+          i = license_images.find(license);
+          if (i != license_images.end())
+            {
+              success(album_id, license, (*i).second, rating);
+              return;
+            }
+          Glib::ustring u = String::ucompose("http://i.creativecommons.org/l/%1/88x31.png", license);
+          Glib::RefPtr < Soup::Message > m = Soup::Message::create ("GET", u);
+          web->queue_message (m, 
+                      sigc::bind(sigc::mem_fun 
+                                 (this, 
+                                  &TrackListLoader::on_license_image_downloaded), 
+                                 album_id, license, rating, success, failure));
+        }
       else
         failure(album_id);
     }
@@ -252,7 +268,25 @@ void TrackListLoader::on_album_info_downloaded(Glib::RefPtr < Soup::Message > &m
     }
 }
 
-bool TrackListLoader::get_license(const char *data, size_t len, Glib::ustring &license)
+void TrackListLoader::on_license_image_downloaded(Glib::RefPtr < Soup::Message > &msg, guint32 album_id, Glib::ustring license, double rating, sigc::slot<void,guint32, Glib::ustring, Glib::RefPtr<Gdk::Pixbuf>, double> success, sigc::slot<void,guint32> failure)
+{
+  if (msg->property_status_code () == 200)
+    {
+      Soup::MessageBody body (msg->gobj ()->response_body);
+      body.set_accumulate (true);
+      Soup::Buffer buffer = body.flatten ();
+      const char* data = buffer.gobj ()->data;
+      int len = buffer.gobj ()->length;
+      Glib::RefPtr<Gio::MemoryInputStream> is = Gio::MemoryInputStream::create();
+      is->add_data(data, len);
+      Glib::RefPtr<Gdk::Pixbuf> license_image = Gdk::Pixbuf::create_from_stream(is);
+      license_images[license] = license_image;
+      success(album_id, license, license_image, rating);
+    }
+  else
+    failure(album_id);
+}
+bool TrackListLoader::get_license_and_rating(const char *data, size_t len, Glib::ustring &license, double &rating)
 {
   Glib::ustring match = 
     "\\<a rel='license' href=\"http://creativecommons.org/licenses/";
@@ -266,5 +300,13 @@ bool TrackListLoader::get_license(const char *data, size_t len, Glib::ustring &l
   if (b[0].size() > 100)
     return false;
   license = b[0];
+  match = "\\<td class='t'\\>Rating\\<\\/td\\> \\<td class='s'\\> ";
+  a = Glib::Regex::split_simple (match, std::string(data));
+  if (a.size () > 1)
+    {
+      b = Glib::Regex::split_simple ("\\<", a[1]);
+      if (b.size() > 0)
+        rating = atof(std::string(b[0]).c_str());
+    }
   return true;
 }
