@@ -56,6 +56,7 @@ VCMAWindow::VCMAWindow(BaseObjectType* baseObject,
 
   set_icon_from_file(Main::get_data_path() + "/video-clip-music-assist.png");
   xml->get_widget("treeview", treeview);
+  treeview->set_has_tooltip(true);
   treeview->signal_button_press_event().connect_notify
     (sigc::mem_fun(*this, &VCMAWindow::on_treeview_clicked));
   songs_list = Gtk::ListStore::create(songs_columns);
@@ -82,6 +83,10 @@ VCMAWindow::VCMAWindow(BaseObjectType* baseObject,
 
   treeview->signal_row_activated().connect
     (sigc::mem_fun(*this, &VCMAWindow::on_song_activated));
+  treeview->get_selection()->signal_changed().connect
+    (sigc::mem_fun(*this, &VCMAWindow::on_song_selected));
+  treeview->signal_query_tooltip().connect
+    (sigc::mem_fun(*this, &VCMAWindow::on_show_treeview_tooltip));
 
   Gtk::Menu::MenuList& menulist = treeview_context_menu.items();
   menulist.push_back
@@ -100,7 +105,6 @@ VCMAWindow::VCMAWindow(BaseObjectType* baseObject,
   treeview->get_column(2)->set_fixed_width(100);
   treeview->get_column(2)->set_max_width(100);
   treeview->get_column(2)->set_expand(false);
-  treeview->set_tooltip_column(2);
   treeview->get_column(3)->set_sort_column(songs_columns.song_title);
   treeview->get_column(3)->set_min_width(50);
   treeview->get_column(3)->set_max_width(200);
@@ -115,6 +119,10 @@ VCMAWindow::VCMAWindow(BaseObjectType* baseObject,
   xml->get_widget("search_label", search_label);
   xml->get_widget("quit_button", quit_button);
   quit_button->signal_clicked().connect(sigc::mem_fun(*this,&VCMAWindow::on_exit_clicked));
+  xml->get_widget("play_button", play_button);
+  play_button->signal_clicked().connect(sigc::mem_fun(*this,&VCMAWindow::on_play_clicked));
+  xml->get_widget("details_button", details_button);
+  details_button->signal_clicked().connect(sigc::mem_fun(*this,&VCMAWindow::on_details_clicked));
   xml->get_widget("minute_spinbutton", minute_spinbutton);
   minute_spinbutton->signal_changed().connect
     (sigc::mem_fun(*this, &VCMAWindow::on_minute_spinbutton_changed));
@@ -141,7 +149,11 @@ VCMAWindow::VCMAWindow(BaseObjectType* baseObject,
   video_file_box->drag_dest_set(targets);
   video_file_box->signal_drag_data_received().connect
     (sigc::mem_fun(*this, &VCMAWindow::on_drag_data_received));
+  treeview->enable_model_drag_source(targets, Gdk::MODIFIER_MASK, Gdk::ACTION_COPY);
+  treeview->signal_drag_data_get().connect
+    (sigc::mem_fun(*this, &VCMAWindow::on_song_drag_data_get));
   xml->get_widget("statusbar", statusbar);
+  update_buttons();
 }
 
 void VCMAWindow::on_window_loaded()
@@ -198,6 +210,37 @@ void VCMAWindow::on_exit_clicked()
   quit();
 }
 
+void VCMAWindow::on_play_clicked()
+{
+  on_play_song_clicked();
+}
+
+void VCMAWindow::on_details_clicked()
+{
+  on_view_album_clicked();
+}
+
+void VCMAWindow::on_song_selected()
+{
+  update_buttons();
+}
+
+void VCMAWindow::update_buttons()
+{
+  Glib::RefPtr<Gtk::TreeSelection> selection = treeview->get_selection();
+  Gtk::TreeModel::iterator iterrow = selection->get_selected();
+  if (iterrow)
+    {
+      play_button->set_sensitive(true);
+      details_button->set_sensitive(true);
+    }
+  else
+    {
+      play_button->set_sensitive(false);
+      details_button->set_sensitive(false);
+    }
+}
+
 void VCMAWindow::on_song_activated(const Gtk::TreeModel::Path& path,
                                    Gtk::TreeViewColumn *col)
 {
@@ -242,7 +285,6 @@ void VCMAWindow::on_about_clicked()
   return;
 }
 
-
 void VCMAWindow::on_view_album_clicked()
 {
   Glib::RefPtr<Gtk::TreeSelection> selection = treeview->get_selection();
@@ -276,6 +318,7 @@ void VCMAWindow::on_search_clicked()
       contents->set_sensitive(false);
       show_spinner();
       songs_list->clear();
+      update_buttons();
       guint32 secs = (minute_spinbutton->get_value() * 60) + second_spinbutton->get_value();
       if (d_connected == false)
         Glib::signal_idle().connect_once(sigc::bind(sigc::mem_fun(*this, &VCMAWindow::load_catalogue_in_the_background), d_track_list, secs));
@@ -570,3 +613,67 @@ void VCMAWindow::on_video_file_selected(Glib::ustring filename)
         }
     }
 }
+    
+void VCMAWindow::on_song_drag_data_get(const Glib::RefPtr<Gdk::DragContext> &context, Gtk::SelectionData &data, guint info, guint time)
+{
+  context->get_source_window()->show();
+  Glib::RefPtr<Gtk::TreeSelection> selection = treeview->get_selection();
+  Gtk::TreeModel::iterator iterrow = selection->get_selected();
+  if (iterrow)
+    {
+      Gtk::TreeModel::Row row = *iterrow;
+      guint32 track_id = row[songs_columns.track_id];
+      data.set_text(String::ucompose("http://www.jamendo.com/get/track/id/track/audio/redirect/%1/?aue=ogg2", track_id));
+    }
+}
+    
+bool VCMAWindow::on_show_treeview_tooltip(int x, int y, bool keyboard_tooltip, const Glib::RefPtr<Gtk::Tooltip> &tooltip)
+{
+  int orig_x = x;
+  int orig_y = y;
+  Gtk::TreeModel::Path path;
+  Gtk::TreeViewColumn *column = NULL;
+  Gtk::TreeModel::iterator i;
+  int cell_x = 0, cell_y = 0;
+  treeview->get_path_at_pos(orig_x, orig_y, path, column, cell_x, cell_y);
+  if (treeview->get_tooltip_context_iter(x, y, keyboard_tooltip, i))
+    {
+      if (!i)
+        return false;
+      if (column == treeview->get_column(1)) //license
+        {
+          Glib::ustring l = (*i)[songs_columns.license];
+          std::vector < Glib::ustring > a = 
+            Glib::Regex::split_simple ("/", l);
+          if (a.size () <= 1)
+            return false;
+          Glib::ustring license;
+          license = "CC";
+          std::vector < Glib::ustring > b = 
+            Glib::Regex::split_simple ("-", a[0]);
+          for (std::vector<Glib::ustring>::iterator j = b.begin();  
+               j != b.end(); j++)
+            license = license + " " + *j;
+
+          license = license + " " + a[1];
+          tooltip->set_text(license);
+          return true;
+        }
+      else if (column == treeview->get_column(2)) //genre
+        {
+          tooltip->set_text((*i)[songs_columns.genre]);
+          return true;
+        }
+      else if (column == treeview->get_column(3)) //song
+        {
+          tooltip->set_text((*i)[songs_columns.song_title]);
+          return true;
+        }
+      else if (column == treeview->get_column(4)) //rating
+        {
+          tooltip->set_text(Glib::ustring::format(std::fixed, std::setprecision(2), (*i)[songs_columns.album_rating] / 2) + " stars");
+          return true;
+        }
+    }
+  return false;
+}
